import { Component, OnInit, Output, Input } from '@angular/core';
import { EventEmitter } from '@angular/core';
import { differenceInCalendarDays, endOfMonth, endOfYear, startOfDay, format, endOfDay, startOfMonth, startOfYear } from "date-fns";
import { DateService } from 'src/app/services/date.service';

@Component({
  selector: 'app-custom-change-date',
  templateUrl: './custom-change-date.component.html',
  styleUrls: ['./custom-change-date.component.less']
})
export class CustomChangeDateComponent implements OnInit {
  @Output() customChangeDateEmit = new EventEmitter();
  @Input() queryType: number;
  constructor(
    private dateService: DateService,
  ) { }

  dateType: string = "day"; //选中时间维度
  dateTypes: any = [
    {
      label: "按天",
      value: "day",
    },
    {
      label: "按月",
      value: "month",
    },
    {
      label: "按年",
      value: "year",
    },
  ]; //时间维度
  defRange = "1"; //默认上周
  defRanges = [
    {
      label: "上周",
      id: 1,
    },
    {
      label: "上月",
      id: 2,
    },
    {
      label: "上年",
      id: 3,
    },
  ]; //时间默认区间

  dateRange: any = []; //发起时间区间 天
  monthRangeStart: any; //发起时间 月 起
  monthRangeEnd: any; //发起时间 月 止
  yearRangeStart: any; //发起时间 年 起
  yearRangeEnd: any; //发起时间 年 止

  ngOnInit() {}

  // this.customChangeDateEmit.emit({queryType, hospital, duty, parent});

  // 修改时间展示维度
  changeDateType(res) {
    console.log(res, this.dateType);
    this.dateType = res;
    switch (res) {
      case "day":
        this.defRanges = [
          {
            label: "上周",
            id: 1,
          },
          {
            label: "上月",
            id: 2,
          },
          {
            label: "上年",
            id: 3,
          },
        ]; //时间默认区间
        this.defRange = "1"; //默认上周
        this.changeDateRange("1");
        break;
      case "month":
        this.defRanges = [
          {
            label: "上月",
            id: 2,
          },
          {
            label: "上年",
            id: 3,
          },
        ]; //时间默认区间
        this.defRange = "2"; //上月
        this.changeDateRange("2");
        break;
      case "year":
        this.defRanges = [
          {
            label: "上年",
            id: 3,
          },
        ]; //时间默认区间
        this.defRange = "3"; //默认上周
        this.changeDateRange("3");
        break;
    }
  }

  // 禁选日期
  disabledDate = (current: Date): boolean => {
    // Can not select days before today and today
    return differenceInCalendarDays(current, this.today) > 0;
  };

  // 禁选月份开始
  disabledMonthStart = (current: Date): boolean => {
    // Can not select days before today and today
    let cur = differenceInCalendarDays(current, endOfMonth(this.today)) > 0;
    let staEnd = differenceInCalendarDays(current, this.monthRangeEnd) > 0;
    return cur || staEnd;
  };
  // 禁选月份结束
  disabledMonthEnd = (current: Date): boolean => {
    // Can not select days before today and today
    let cur = differenceInCalendarDays(current, endOfMonth(this.today)) > 0;
    let staEnd = differenceInCalendarDays(this.monthRangeStart, current) > 0;
    return cur || staEnd;
  };

  // 禁选年份开始
  disabledYearStart = (current: Date): boolean => {
    // Can not select days before today and today
    let cur = differenceInCalendarDays(current, endOfYear(this.today)) > 0;
    let staEnd = differenceInCalendarDays(current, this.yearRangeEnd) > 0;
    return cur || staEnd;
  };

  // 禁选年份结束
  disabledYearEnd = (current: Date): boolean => {
    // Can not select days before today and today
    let cur = differenceInCalendarDays(current, endOfYear(this.today)) > 0;
    let staEnd = differenceInCalendarDays(this.yearRangeStart, current) > 0;
    return cur || staEnd;
  };

  // 日期选择 日
  startDate: string; //发起时间开始
  endDate: string; //发起时间结束
  changeDate(result?): void {
    console.log(this.dateRange);
    console.log(result);
    this.dateRange = result;
    if (!this.quick) {
      // 不是快捷选择
      this.defRange = null;
    }
    if (!result || !result.length) {
      this.startDate = this.endDate = "";
      return;
    }
    this.startDate =
      result[0].getFullYear() +
      "-" +
      (result[0].getMonth() + 1) +
      "-" +
      result[0].getDate();
    this.endDate =
      result[1].getFullYear() +
      "-" +
      (result[1].getMonth() + 1) +
      "-" +
      result[1].getDate();
  }

  // 月份选择
  changeMonthStart(result?) {
    console.log(result);
    this.monthRangeStart = result;
    if (!this.quick) {
      // 不是快捷选择
      this.defRange = null;
    }
    if (!result) {
      this.startDate = this.endDate = "";
      return;
    }
    this.startDate = format(startOfMonth(result), 'yyyy-MM-dd');
  }
  changeMonthEnd(result?) {
    console.log(result);
    this.monthRangeEnd = result;
    if (!this.quick) {
      // 不是快捷选择
      this.defRange = null;
    }
    if (!result) {
      this.startDate = this.endDate = "";
      return;
    }
    this.endDate = format(endOfMonth(result), 'yyyy-MM-dd');
  }
  // 年份选择
  changeYearStart(result?) {
    console.log(result);
    this.yearRangeStart = result;
    if (!this.quick) {
      // 不是快捷选择
      this.defRange = null;
    }
    if (!result) {
      this.startDate = this.endDate = "";
      return;
    }
    this.startDate = format(startOfYear(result), 'yyyy-MM-dd');
    // this.endDate = result.getFullYear() + '-01-01';
  }
  changeYearEnd(result?) {
    console.log(result);
    this.yearRangeEnd = result;
    if (!this.quick) {
      // 不是快捷选择
      this.defRange = null;
    }
    if (!result) {
      this.startDate = this.endDate = "";
      return;
    }
    this.endDate = format(endOfYear(result), 'yyyy-MM-dd');
  }

  // 日期选择 快速修改时间区间
  today = new Date();
  quick: boolean = false;
  changeDateRange(res) {
    console.log(res);
    this.quick = true;
    switch (res) {
      case "1":
        // 上周
        let lastweekstartdate = this.dateService.date().lastWeekStartDate;
        let lastweekenddate = this.dateService.date().lastWeekEndDate;
        console.log(lastweekstartdate, lastweekenddate);
        this.changeDate([lastweekstartdate, lastweekenddate]);
        break;
      case "2":
        // 上月
        let lastmonthstartdate = this.dateService.date().lastMonthStartDate;
        let lastmonthenddate = this.dateService.date().lastMonthEndDate;
        console.log(lastmonthstartdate, lastmonthenddate);
        this.changeDate([lastmonthstartdate, lastmonthenddate]);
        this.changeMonthStart(lastmonthstartdate);
        this.changeMonthEnd(lastmonthenddate);
        break;
      case "3":
        // 上年
        let lastyearstartdate = this.dateService.date().lastYearStartDate;
        let lastyearenddate = this.dateService.date().lastYearEndDate;
        console.log(lastyearstartdate, lastyearenddate);
        this.changeDate([lastyearstartdate, lastyearenddate]);
        this.changeMonthStart(lastyearstartdate);
        this.changeMonthEnd(lastyearenddate);
        this.changeYearStart(lastyearstartdate);
        this.changeYearEnd(lastyearenddate);
        break;
    }
    this.quick = false;
  }
}
